export NP_DO_TEST=1
export NP_DO_TEST6=1
export NP_DEBUG=0

export NP_LO_DEV=lo0
export NP_LO_IP=localhost
export NP_LO_IP6=localhost
export NP_LO_TARGET_IP=localhost
export NP_LO_TARGET_IP6=localhost
export NP_LO_TARGET_PORT=22

export NP_ETH_DEV=vr0
export NP_ETH_IP=dani
export NP_ETH_IP6=dani
export NP_ETH_MAC=00:50:ba:a1:5f:8a
export NP_ETH_TARGET_IP=cadaveria
export NP_ETH_TARGET_IP6=cadaveria
export NP_ETH_TARGET_PORT=23
